// JavaScript script for Adobe Illustrator
// Tested with Adobe Illustrator CS3 13.0.3, Windows XP SP2 (Japanese version).
// This script provided "as is" without warranty of any kind.
// Free to use and distribute.

// 2009-06-09
// Copyright(c) 2009 Hiroyuki Sato
// http://lines-about-to-be-generated.blogspot.com/

// settings ---------------------------
const SR = 100; // radius of the sphere
const R  = 12;   // radius of the circles on the sphere

// number of the circles for each surface (front/back)
const NUM_CIRCLES = 50;

// fill color for the circles
const GRAY_LEVEL_FRONT  = 80;
const GRAY_LEVEL_BACK   = 20;
const OPACITY_VALUE     = 80.0;

const FOCAL_LENGTH = 1600;

// lower limit for the height of the ellipse
const HEIGHT_LIMIT = 1;

const DRAW_AT_ARTBOARD_ORIGIN = false;
// ------------------------------------

var o = [0, 0];
if( ! DRAW_AT_ARTBOARD_ORIGIN ){
  o = [activeDocument.width / 2, activeDocument.height / 2];
}

var lay = activeDocument.activeLayer;

var h_limit = HEIGHT_LIMIT / R;

circleOnSphere( GRAY_LEVEL_BACK, -1); // far side
circleOnSphere( GRAY_LEVEL_FRONT, 1); // this side

// ------------------------------------
function circleOnSphere(gray_level, sign){
  var col = new GrayColor();
  col.gray = gray_level;
  
  var gr = lay.groupItems.add();
  var c, s; // cos, sin
  var pi, r2;
  
  for(var i = 0; i < NUM_CIRCLES; i++){
    // gets random value (from h_limit to 1), as cosine
    c = Math.random() * (1 - h_limit) + h_limit;
    
    s = Math.sqrt(1 - c * c);
    z = sign * c * SR;

    r2 = getPers(R, z);
    
    pi = lay.pathItems.ellipse(
      r2, getPers(s * SR - c * R, z),
      r2 * 2 * c, r2 * 2);
    
    with(pi){
      filled = true;
      fillColor = col;
      stroked = false;
      opacity = OPACITY_VALUE;
      
      rotate(Math.random() * 360, true,true,true,true,
             Transformation.DOCUMENTORIGIN);
      translate(o[0], o[1]);
      move(gr, ElementPlacement.PLACEATBEGINNING);
    }
  }
}

// ------------------------------------
function getPers(x, z){
  return x * FOCAL_LENGTH / (FOCAL_LENGTH - z);
}
